import java.io.IOException;
import java.io.FileWriter;
import java.io.PrintWriter;

/**
 * Esempi di catch di eccezioni.
 * 
 * @author Stefano Guerrini
 * @version Marzo 2009
 */
public final class ProvaEccezioni
{
    /**
     * Questa classe non e' istanziabile.
     * Nascondo il costruttore
     */
    private ProvaEccezioni() {}

    /**
     * Una funzione per generare vettori di interi lunghi size
     * Il vettore contiene i numeri da 0 a size-1
     * 
     * @param size      dimensione del vettore da generare
     * @return          il vettore
     */
    public static int[] generaArray(int size)
    {
        int v[] = new int[size];
        for (int i = 0; i < size; i++)
            v[i] = i;
        return v;
    }
    
    /**
     * Stampa un array di interi su di un file
     * 
     * Semplice esempio di gestione delle eccezioni
     * 
     * Cattura eccezioni checked e unchecked e
     * mostra utilizzo di finally
     * 
     * @param fileName      nome del file su cui stampare
     * @param v             vettore da stampare
     * @param len           numero di elementi da stampare
     * @param rows          numero di righer piene da stampare
     */
    public static void writeArray(String fileName, int v[], int len, int rows)
    {                                   
        System.out.println("  Entering writeArray");
        
        PrintWriter out = null;
    
        try {
            System.out.println("  Entering try statement");

            int n = len / rows; // elementi per riga

            // apre il file di stampa usando un metodo
            // che permette la scrittura di caratteri
            out = new PrintWriter(new FileWriter(fileName));

            // stampa gli elementi
            for (int i = 0, j = 0; j < len; j++) {
                System.out.print(v[j]);
                out.print(v[j]);
                i = (i+1) % n; // conta elementi della riga
                // separa gli elementi o chiudi riga
                out.print(i == 0 ? "\n" : " ");
                System.out.print(i == 0 ? "\n" : " ");
            }
            // chiudi ultima riga se necessario
            if (len % n != 0) {
                out.println();
                System.out.println();
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            System.out.println("  Entering catch ArrayIndexOutOfBoundsException");
            System.err.println("Caught "
                        + "ArrayIndexOutOfBoundsException: " 
                        + e.getMessage());
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("  Entering catch IOException");
            System.err.println("Caught IOException: "  
                        +  e.getMessage());
            e.printStackTrace();
        } finally {
            System.out.println("  Entering finally");
            if (out != null) {
                System.out.println("    Closing PrintWriter");
                out.close();
            } else {
                System.out.println("    PrintWriter not open");
            }
        }
    }
    
    /**
     * Chiama la writeArray catturando una eventuale eccezione aritmetica
     * 
     * Ha gli stessi parametri della writeArray
     * 
     * Utile per verificare la divisione per 0 
     * che si verifica in writeArray se rows = 0
     */
    public static void writeArrayBis(String fileName, int v[], int len, int rows)
    {
        System.out.println("Entering writeArrayBis");

        try {
            System.out.println("Entering try statement");
            writeArray(fileName, v, len, rows);
        } catch (ArithmeticException e) {
            System.out.println("Entering catch ArithmeticException");
            System.err.println("Caught "
                        + "ArithmeticException: " 
                        + e.getMessage());
            e.printStackTrace();
        }
    }
  
    /**
     * Funzione di prova per verificare errori di I/O
     * dovuti all'accessibilita' al file
     * Utilizza vettore, dim del vettore e righe corretti
     * 
     * @param fileName      nome del file su cui provare a scrivere
     */
    public static void provaFileAccess(String fileName)
    {
        int size = 10;
        int rows = 3;
        writeArrayBis(fileName, generaArray(size), size, rows);
    }
    
    /**
     * Funzione di prova che scrive su di un file scrivibile
     */
    public static void provaOK()
    {
        System.out.println("----\n---- Run OK\n----");
        String fileName = "out.txt"; // dovrebbe essere scrivibile
        provaFileAccess(fileName);
    }

    /**
     * Funzione di prova che scrive su di un file non scrivibile
     */
    public static void provaIOException()
    {
        System.out.println("----\n---- Test IOException\n----");
        String fileName = "locked.txt"; // dovrebbe essere protetto in scrittura
        provaFileAccess(fileName);
    }

    /**
     * Funzione di prova che verifica errori nell'accesso
     * al vettore (indice fuori dei limiti)
     * 
     * Esegue due test che causano errori:
     *      il primo per indice maggiore del max,
     *      il secondo per indice minore del min
     */
    public static void provaArrayBoundsException()
    {
        int size = 10;
        int rows = 3;
        int v[] = generaArray(size);
        System.out.println("----\n---- Primo Test ArrayBoundsException\n----");
        writeArrayBis("out1.txt", v, size+1, rows);
        System.out.println("----\n---- Secondo Test ArrayBoundsException\n----");
        writeArrayBis("out2.txt", generaArray(size), 0, rows);
    }

    /**
     * Funzione di prova che genera e cattura
     * un errore di divisione per 0
     */
    public static void provaArithmeticExceptionCaught()
    {
        int size = 10;
        int rows = 0;
        int v[] = generaArray(size);
        System.out.println("----\n---- Test Arithmetic Exception \n----");
        // rows == 0 si avra' una divisione per 0
        writeArrayBis("out1.txt", v, size, rows);
        // L'eccezione  catturata all'interno di writeArrayBis
        // La linea seguente viene eseguita
        System.out.println("provaArithmeticExceptionCaught:");
        System.out.println("  posso continuare, l'eccezione  stata catturata!");        
    }

    /**
     * Funzione di prova che genera e cattura
     * un errore di divisione per 0
     */
    public static void provaArithmeticExceptionUncaught()
    {
        int size = 10;
        int rows = 0;
        int v[] = generaArray(size);
        System.out.println("----\n---- Test Arithmetic Exception \n----");
        // rows == 0 si avra' una divisione per 0
        writeArray("out1.txt", v, size, rows);
        // L'eccezione non  catturata all'interno di writeArray
        // La linea seguente non dovrebbe essere eseguita
        System.out.println("provaArithmeticExceptionUncaught:");
        System.out.println("  questa linea non dovrebbe essere stampata!");        
    }
}
